-- (c) 2006 Witold Bolt (ja@hope.art.pl)

-- funkcja sprawdza czy w strukturze drzewa kategorii nie ma petli ...
CREATE OR REPLACE FUNCTION watchForLoops() RETURNS trigger AS $$
    DECLARE
        parentID integer := NEW.pid;
        childID integer := NEW.id;

    BEGIN

    IF NEW.pid IS NULL THEN
        RETURN NEW;
    END IF;

    LOOP
        IF parentID = NEW.id THEN
            RAISE EXCEPTION 'wykroto petle!';
        END IF;

        childID:=parentID;

        SELECT INTO parentID pid FROM kategoria WHERE id=childID;

        IF NOT FOUND THEN
            RAISE EXCEPTION 'podano nieistniejacy parentID';
        END IF;

        IF parentID IS NULL THEN
            RETURN NEW;
        END IF;

    END LOOP;

    RETURN NEW;

    END;

$$ LANGUAGE plpgsql;


-- funkcja poprawia drzewo kategorii w momencie kasowania rekordu
CREATE OR REPLACE FUNCTION fixTree() RETURNS trigger AS $$
    BEGIN

    IF OLD.pid IS NOT NULL THEN
        UPDATE kategoria SET pid=OLD.pid WHERE pid=OLD.id;
    ELSE
        UPDATE kategoria SET pid=NULL WHERE pid=OLD.id;
    END IF;

    RETURN OLD;

    END;
$$ LANGUAGE plpgsql;

-- historiaInsertWatch

CREATE OR REPLACE FUNCTION historiaInsertWatch() RETURNS trigger AS $$
    BEGIN

-- czy uzytkownik ma prawo do wypozyczenia ksiazki?
    IF NEW.uid NOT IN (SELECT * FROM getEgzemplarzAllowedUsers(NEW.eid)) THEN
        RAISE EXCEPTION 'Uzytkownik nie ma prawa do wypozyczenia tej ksiazki';
    END IF;

-- czy ksiazka nie jest juz wypozyczona
    IF NEW.eid IN (SELECT eid FROM wypozyczenie) THEN
        RAISE EXCEPTION 'Ksiazka jest juz wypozyczona';
    END IF;

-- czy ksiazka nie ma ustawionej flagi ze nie mozna wypozyczac
    IF mozna_wypozyczyc(NEW.eid) = FALSE THEN
        RAISE EXCEPTION 'Wlasciciel ksiazki nie chce jej wypozyczac!';
    END IF;

-- czy ktos nie chce wpisac za duzo (insert odbywa sie w momencie wypozyczenia!)
    IF NEW.data_zwrotu IS NOT NULL THEN
        RAISE EXCEPTION 'Nie mozna wstawiac do historii rekordow z data_zwrotu';
    END IF;

-- czy nie wypozyczamy sami od siebie
    IF NEW.uid IN (SELECT uid FROM egzemplarz WHERE id=NEW.eid) THEN
        RAISE EXCEPTION 'Bez sensu. Nie wypozyczaj sam od siebie!';
    END IF;

    RETURN NEW;

    END;

$$ LANGUAGE plpgsql;

CREATE TRIGGER watchForLoops BEFORE INSERT OR UPDATE ON kategoria
    FOR EACH ROW EXECUTE PROCEDURE watchForLoops();

CREATE TRIGGER fixTree BEFORE DELETE ON kategoria
    FOR EACH ROW EXECUTE PROCEDURE fixTree();

CREATE TRIGGER historiaInsertWatch BEFORE INSERT ON historia
    FOR EACH ROW EXECUTE PROCEDURE historiaInsertWatch();
